@@ -2,8 +2,8 @@ package Inline;
 
 use strict;
 require 5.006;
-$Inline::VERSION = '0.55';
-$Inline::VERSION = eval $Inline::VERSION;
+our $VERSION = '0.56';
+$VERSION = eval $VERSION;
 
 use AutoLoader 'AUTOLOAD';
 use Inline::denter;
@@ -514,16 +514,7 @@ sub check_installed {
 sub load {
     my $o = shift;
 
-    if ($o->{CONFIG}{_INSTALL_}) {
-	my $inline = "$o->{API}{modfname}.inl";
-	open INLINE, "> $inline"
-	  or croak M24_open_for_output_failed($inline);
-	print INLINE "*** AUTOGENERATED by Inline.pm ***\n\n";
-	print INLINE "This file satisfies the make dependency for ";
-	print INLINE "$o->{API}{modfname}.pm\n";
-	close INLINE;
-	return;
-    }
+    return if $o->{CONFIG}{_INSTALL_};
 
     my ($pkg, $module) = @{$o->{API}}{qw(pkg module)};
     croak M42_usage_loader() unless $o->{INLINE}{ILSM_type} eq 'compiled';
@@ -548,6 +539,23 @@ END
     croak M43_error_bootstrap($module, $@) if $@;
 }
 
+#==============================================================================
+# Create file that satisfies the Makefile dependency for this object
+#==============================================================================
+
+sub satisfy_makefile_dep {
+    my $o = shift;
+
+       my $inline = $o->{API}{modinlname};
+       open INLINE, "> $inline"
+         or croak M24_open_for_output_failed($inline);
+       print INLINE "*** AUTOGENERATED by Inline.pm ***\n\n";
+       print INLINE "This file satisfies the make dependency for ";
+       print INLINE "$o->{API}{module}\n";
+       close INLINE;
+       return;
+}
+
 #==============================================================================
 # Process the config options that apply to all Inline sections
 #==============================================================================
@@ -838,6 +846,8 @@ sub create_config_file {
 		next;
 	    }
 	    next if $mod =~ /^(MakeMaker|denter|messages)$/;
+	    # @INC is made safe by -T disallowing PERL5LIB et al
+	    ($mod) = $mod =~ /(.*)/;
 	    eval "require Inline::$mod;";
             warn($@), next if $@;
 	    eval "\$register=&Inline::${mod}::register";
@@ -983,6 +993,7 @@ sub install {
     my @modparts = split(/::/,$o->{API}{module});
     $o->{API}{modfname} = $modparts[-1];
     $o->{API}{modpname} = File::Spec->catdir(@modparts);
+    $o->{API}{modinlname} = join('-',@modparts).'.inl';
     $o->{API}{suffix} = $o->{INLINE}{ILSM_suffix};
     $o->{API}{build_dir} = File::Spec->catdir($o->{INLINE}{DIRECTORY},'build',
                                               $o->{API}{modpname});
@@ -1037,11 +1048,9 @@ sub with_configs {
     my $o = shift;
     my @configs;
     for my $mod (@{$o->{CONFIG}{WITH}}) {
-	my $ref = eval {
-	    no strict 'refs';
-	    &{$mod . "::Inline"}($o->{API}{language});
-	};
+	my $ref = eval { $mod->Inline($o->{API}{language}); };
 	croak M25_no_WITH_support($mod, $@) if $@;
+	croak M65_WITH_not_lang($mod, $o->{API}{language}) unless $ref;
 	push @configs, %$ref;
     }
     return @configs;
@@ -1061,19 +1070,21 @@ sub env_untaint {
       }
     }
 
+    # only accept dirs that are absolute and not world-writable
     $ENV{PATH} = $^O eq 'MSWin32' ?
                  join ';', grep {not /^\./ and -d $_
 				  } split /;/, $ENV{PATH}
                  :
-                 join ':', grep {not /^\./ and -d $_ and not -w $_ || -W $_
+                 join ':', grep {/^\// and -d $_ and $< == $> ? 1 : not (-W $_ or -O $_)
                                   } split /:/, $ENV{PATH};
-# Was:
-#                join ':', grep {not /^\./ and -d $_ and
-#		                 not ((stat($_))[2] & 0022)
-#				  } split /:/, $ENV{PATH};
 
     map {($_) = /(.*)/} @INC;
 
+    # list cherry-picked from `perldoc perlrun`
+    delete @ENV{qw(PERL5OPT PERL5SHELL PERL_ROOT IFS CDPATH ENV BASH_ENV)};
+    $ENV{SHELL} = '/bin/sh' if -x '/bin/sh';
+
+    $< = $> if $< != $>; # so child processes retain euid - ignore failure
 }
 #==============================================================================
 # Blindly untaint tainted fields in Inline object.
@@ -1082,6 +1093,7 @@ sub obj_untaint {
     my $o = shift;
     warn "In Inline::obj_untaint() : Blindly untainting tainted fields in Inline object.\n" unless $o->{CONFIG}{NO_UNTAINT_WARN};
     ($o->{INLINE}{ILSM_module}) = $o->{INLINE}{ILSM_module} =~ /(.*)/;
+    ($o->{API}{directory}) = $o->{API}{directory} =~ /(.*)/;
     ($o->{API}{build_dir}) = $o->{API}{build_dir} =~ /(.*)/;
     ($o->{CONFIG}{DIRECTORY}) = $o->{CONFIG}{DIRECTORY} =~ /(.*)/;
     ($o->{API}{install_lib}) = $o->{API}{install_lib} =~ /(.*)/;
@@ -1240,7 +1252,7 @@ END
 }
 
 #==============================================================================
-# Hand off this invokation to Inline::MakeMaker
+# Hand off this invocation to Inline::MakeMaker
 #==============================================================================
 sub maker_utils {
     require Inline::MakeMaker;
@@ -2042,5 +2054,12 @@ Only C and CPP (C++) based modules are currently supported.
 END
 }
 
+sub M65_WITH_not_lang {
+    my ($mod, $lang) = @_;
+    return <<END;
+$mod gave no 'with' hints for $lang.
+END
+}
+
 1;
 __END__
