@@ -1,9 +1,9 @@
 use strict;
 package Inline::Foo;
+$Inline::Foo::VERSION = '0.01';
 
 require Inline;
-our @ISA = qw(Inline);
-our $VERSION = '0.85_003';
+@Inline::Foo::ISA = qw(Inline);
 
 use Carp;
 use File::Spec;
@@ -54,7 +54,6 @@ sub build {
     $code =~ s/bar-//g if $o->{ILSM}{BAR};
     {
         package Foo::Tester;
-        our $VERSION = '0.02';
         eval $code;
     }
     croak "Foo build failed:\n$@" if $@;
@@ -63,7 +62,6 @@ sub build {
     $o->mkpath($path) unless -d $path;
     open FOO_OBJ, "> $obj"
       or croak "Can't open $obj for output\n$!";
-    binmode(FOO_OBJ, ':utf8');
     print FOO_OBJ $code;
     close \*FOO_OBJ;
 }
@@ -73,7 +71,6 @@ sub load {
     my $obj = $o->{API}{location};
     open FOO_OBJ, "< $obj"
       or croak "Can't open $obj for output\n$!";
-    binmode(FOO_OBJ, ':utf8');
     my $code = join '', <FOO_OBJ>;
     close \*FOO_OBJ;
     eval "package $o->{API}{pkg};\n$code";
